'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _alinexUtil = require('alinex-util');

var _alinexUtil2 = _interopRequireDefault(_alinexUtil);

var _child_process = require('child_process');

var _child_process2 = _interopRequireDefault(_child_process);

var _es6Promisify = require('es6-promisify');

var _es6Promisify2 = _interopRequireDefault(_es6Promisify);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _Data = require('./Data');

var _Data2 = _interopRequireDefault(_Data);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

let format = null;

const exec = (0, _es6Promisify2.default)(_child_process2.default.exec);
const readFile = (0, _es6Promisify2.default)(_fs2.default.readFile);

function sourceFunction(data) {
  return typeof data === 'function' ? data() : data;
}

function sourceEnvironment(data) {
  if (typeof data !== 'string' || !_alinexUtil2.default.string.starts(data, 'env://')) return data;
  return process.env[data.substring(6)];
}

function sourceCommand(data) {
  if (typeof data !== 'string' || !_alinexUtil2.default.string.starts(data, 'exec://')) return data;
  return exec(data.substring(7));
}

function sourceSsh(data) {
  if (typeof data !== 'string' || !_alinexUtil2.default.string.starts(data, 'ssh://')) return data;
  return 'xxx';
}

function sourceFile(data) {
  if (typeof data !== 'string' || !_alinexUtil2.default.string.starts(data, 'file://')) return data;
  return readFile(data.substring(7), 'utf8');
}

function sourceWeb(data) {
  if (typeof data !== 'string' || !data.match(/https?:\/\//)) return data;
  return Promise.resolve().then(() => require('request-promise-native')).then(request => request(data));
}

const accessor = {};

class Reference {

  constructor(base) {
    if (base) this.base = base;
    this.access = [];
    this._raw = false;
  }

  inspect(depth, options) {
    const newOptions = Object.assign({}, options, {
      depth: options.depth === null ? null : options.depth - 1
    });
    const padding = ' '.repeat(5);
    const base = this.base ? _alinexUtil2.default.inspect(this.base, newOptions).replace(/\n/g, `\n${padding}`) : 'Data';
    const inner = this.access.map(e => _alinexUtil2.default.inspect(e, newOptions).replace(/\n/g, `\n${padding}`));
    inner.unshift(base);
    return `${options.stylize(this.constructor.name, 'class')} ${inner.join(' ➞ ')} `;
  }

  raw(flag = true) {
    this._raw = flag;
    return this;
  }

  path(def) {
    def = def.replace(/\/\.\//g, '/').replace(/^\.\//, '');
    this.access.push(['path', def]);
    return this;
  }

  keys() {
    this.access.push(['keys', true]);
    return this;
  }

  values() {
    this.access.push(['values', true]);
    return this;
  }

  trim() {
    this.access.push(['trim', true]);
    return this;
  }

  split(...def) {
    if (!def.length) def = ['\n'];
    this.access.push(['split', def]);
    return this;
  }

  join(...def) {
    if (!def.length) def = ['\n'];
    this.access.push(['join', def]);
    return this;
  }

  match(def) {
    this.access.push(['match', def]);
    return this;
  }

  range(...def) {
    this.access.push(['range', def]);
    return this;
  }

  filter(...def) {
    let opt = def;
    if (def[0] instanceof RegExp) {
      ;
      opt = def[0];
    }this.access.push(['filter', opt]);
    return this;
  }

  exclude(...def) {
    let opt = def;
    if (def[0] instanceof RegExp) {
      ;
      opt = def[0];
    }this.access.push(['exclude', opt]);
    return this;
  }

  parse(def) {
    this.access.push(['parse', def]);
    return this;
  }

  fn(def) {
    this.access.push(['fn', def]);
    return this;
  }

  or(def) {
    this.access.push(['or', def]);
    return this;
  }

  concat(def) {
    this.access.push(['concat', def]);
    return this;
  }

  get description() {
    let msg = `reference at ${_alinexUtil2.default.inspect(this.base)}`;
    if (this.access.length) msg += ` -> ${this.access.join(' -> ')}`;
    return msg;
  }

  resolve(pos) {
    let p = Promise.resolve(this.base || pos).then(data => sourceFunction(data)).then(data => sourceEnvironment(data)).then(data => sourceCommand(data)).then(data => sourceSsh(data)).then(data => sourceFile(data)).then(data => sourceWeb(data));

    this.access.forEach(([fn, def]) => {
      p = p.then(data => accessor[fn](data, def, pos));
    });
    return p.then(data => data instanceof _Data2.default ? data.value : data).catch(err => err instanceof Error ? Promise.reject(err) : err);
  }
}

accessor.path = (data, def) => {
  if (typeof data !== 'object') return data;

  if (data instanceof _Data2.default) {
    while (def[0] === '/' || def[0] === '.') {
      if (def[0] === '/') {
        def = def.substring(1);
        data = data.root;
      } else if (_alinexUtil2.default.string.starts(def, '../')) {
        def = def.substring(3);
        data = data.parent;
      }
    }
    data = data.value;
  }

  def = def.replace(/[^/]+\/\.\.\//g, '').replace(/^(\.{,2}\/)+/, '');
  return _alinexUtil2.default.object.pathSearch(data, def);
};

accessor.keys = data => {
  if (typeof data !== 'object') return data;
  return Object.keys(data);
};

accessor.values = data => {
  if (typeof data !== 'object') return data;
  return Object.values(data);
};

accessor.trim = data => {
  if (typeof data === 'string') return data.trim();
  if (Array.isArray(data)) return data.map(e => accessor.trim(e));
  if (typeof data === 'object') {
    const obj = {};
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = Object.keys(data)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        const key = _step.value;
        obj[key] = accessor.trim(data[key]);
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return obj;
  }
  return data;
};

accessor.split = (data, def) => {
  if (typeof data === 'string') {
    data = data.split(def[0]);
    if (def[1]) data = data.map(e => e.split(def[1]));
    if (def[2]) data = data.map(e => e.map(f => f.split(def[2])));
    return data;
  }
  if (Array.isArray(data)) return data.map(e => accessor.split(e, def));
  if (typeof data === 'object') {
    const obj = {};
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      for (var _iterator2 = Object.keys(data)[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        const key = _step2.value;
        obj[key] = accessor.split(data[key], def);
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2.return) {
          _iterator2.return();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }

    return obj;
  }
  return data;
};

accessor.join = (data, def) => {
  if (typeof data === 'string') return data;
  if (Array.isArray(data)) {
    if (def[2]) {
      data = data.map(e => Array.isArray(e) ? e.map(f => Array.isArray(f) ? f.join(def[2]) : f) : e);
    }
    if (def[1]) data = data.map(e => Array.isArray(e) ? e.join(def[1]) : e);
    return data.join(def[0]);
  }
  if (typeof data === 'object') {
    const obj = {};
    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
      for (var _iterator3 = Object.keys(data)[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
        const key = _step3.value;
        obj[key] = accessor.join(data[key], def);
      }
    } catch (err) {
      _didIteratorError3 = true;
      _iteratorError3 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion3 && _iterator3.return) {
          _iterator3.return();
        }
      } finally {
        if (_didIteratorError3) {
          throw _iteratorError3;
        }
      }
    }

    return obj;
  }
  return data;
};

accessor.match = (data, def) => {
  if (typeof data === 'string') return data.match(def);
  if (Array.isArray(data)) return data.map(e => accessor.match(e, def));
  if (typeof data === 'object') {
    const obj = {};
    var _iteratorNormalCompletion4 = true;
    var _didIteratorError4 = false;
    var _iteratorError4 = undefined;

    try {
      for (var _iterator4 = Object.keys(data)[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
        const key = _step4.value;
        obj[key] = accessor.match(data[key], def);
      }
    } catch (err) {
      _didIteratorError4 = true;
      _iteratorError4 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion4 && _iterator4.return) {
          _iterator4.return();
        }
      } finally {
        if (_didIteratorError4) {
          throw _iteratorError4;
        }
      }
    }

    return obj;
  }
  return data;
};

accessor.range = (data, def) => {
  if (typeof data !== 'object') return data;
  if (!Array.isArray(data)) data = Object.values(data);
  let obj = [];
  var _iteratorNormalCompletion5 = true;
  var _didIteratorError5 = false;
  var _iteratorError5 = undefined;

  try {
    for (var _iterator5 = def[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
      const range = _step5.value;

      let end = range[1];
      if (end === undefined) end = range[0] + 1;
      if (end === 0) end = data.length;
      obj = obj.concat(data.slice(range[0], end));
    }
  } catch (err) {
    _didIteratorError5 = true;
    _iteratorError5 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion5 && _iterator5.return) {
        _iterator5.return();
      }
    } finally {
      if (_didIteratorError5) {
        throw _iteratorError5;
      }
    }
  }

  return obj;
};

accessor.filter = (data, def) => {
  if (data === undefined) return data;
  if (typeof data !== 'object') {
    if (def instanceof RegExp) return data.toString().match(def) ? def : undefined;
    return def.includes(data.toString()) ? data : undefined;
  }
  if (!Array.isArray(data)) data = Object.values(data);
  const obj = [];
  var _iteratorNormalCompletion6 = true;
  var _didIteratorError6 = false;
  var _iteratorError6 = undefined;

  try {
    for (var _iterator6 = data[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
      let check = _step6.value;

      if (typeof check !== 'string') check = check.toString();
      if (def instanceof RegExp) {
        if (check.match(def)) obj.push(check);
      } else if (def.includes(check)) obj.push(check);
    }
  } catch (err) {
    _didIteratorError6 = true;
    _iteratorError6 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion6 && _iterator6.return) {
        _iterator6.return();
      }
    } finally {
      if (_didIteratorError6) {
        throw _iteratorError6;
      }
    }
  }

  return obj;
};

accessor.exclude = (data, def) => {
  if (data === undefined) return data;
  if (typeof data !== 'object') {
    if (def instanceof RegExp) return data.toString().match(def) ? undefined : def;
    return def.includes(data.toString()) ? undefined : data;
  }
  if (!Array.isArray(data)) data = Object.values(data);
  const obj = [];
  var _iteratorNormalCompletion7 = true;
  var _didIteratorError7 = false;
  var _iteratorError7 = undefined;

  try {
    for (var _iterator7 = data[Symbol.iterator](), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
      let check = _step7.value;

      if (typeof check !== 'string') check = check.toString();
      if (def instanceof RegExp) {
        if (!check.match(def)) obj.push(check);
      } else if (!def.includes(check)) obj.push(check);
    }
  } catch (err) {
    _didIteratorError7 = true;
    _iteratorError7 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion7 && _iterator7.return) {
        _iterator7.return();
      }
    } finally {
      if (_didIteratorError7) {
        throw _iteratorError7;
      }
    }
  }

  return obj;
};

accessor.parse = (data, def) => {
  if (!format) format = require('alinex-format');
  if (typeof data !== 'string') return data;
  const parser = (0, _es6Promisify2.default)(format.parse);
  return parser(data, def);
};


accessor.fn = (data, def) => def(data);

accessor.or = (data, def, pos) => {
  if (data === undefined) return def.resolve(pos);
  return data;
};

accessor.concat = (() => {
  var _ref = _asyncToGenerator(function* (data, def, pos) {
    if (Array.isArray(data)) {
      const sub = yield def.resolve(pos);
      if (Array.isArray(sub)) return data.concat(sub);
    } else if (typeof data === 'object') {
      const sub = yield def.resolve(pos);
      if (typeof sub === 'object') {
        var _iteratorNormalCompletion8 = true;
        var _didIteratorError8 = false;
        var _iteratorError8 = undefined;

        try {
          for (var _iterator8 = Object.keys(sub)[Symbol.iterator](), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
            const key = _step8.value;
            data[key] = sub[key];
          }
        } catch (err) {
          _didIteratorError8 = true;
          _iteratorError8 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion8 && _iterator8.return) {
              _iterator8.return();
            }
          } finally {
            if (_didIteratorError8) {
              throw _iteratorError8;
            }
          }
        }
      }
    }
    return data;
  });

  return function (_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
})();

exports.default = Reference;