'use strict';

var _yargs = require('yargs');

var _yargs2 = _interopRequireDefault(_yargs);

var _alinexCore = require('alinex-core');

var _alinexCore2 = _interopRequireDefault(_alinexCore);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _index = require('./index');

var _index2 = _interopRequireDefault(_index);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let quiet = false;
var _arr = ['--get-yargs-completions', 'bashrc', '-q', '--quiet'];
for (var _i = 0; _i < _arr.length; _i++) {
  const a = _arr[_i];
  if (process.argv.includes(a)) quiet = true;
}

_alinexCore2.default.initExit();
process.on('exit', () => {
  if (!quiet) console.log('Goodbye\n');
});

if (!quiet) {
  console.log(_chalk2.default.grey('Initializing alinex-validator...'));
}

_yargs2.default.usage('\nUsage: $0 [options]').env('VALIDATOR').example('$0 -i config.yml -s schema.js', 'to check the file for validity').example('$0 -i config.yml -s schema.js -o result.json', 'to transform the data to the json file').option('input', {
  alias: 'i',
  describe: 'file or directory to read data from',
  type: 'string',
  array: true,
  demandOption: true
}).option('schema', {
  alias: 's',
  describe: 'schema to use for validation',
  type: 'string',
  demandOption: true
}).option('output', {
  alias: 'o',
  describe: 'file to write resulting data structure to',
  type: 'string'
}).option('force', {
  alias: 'f',
  describe: 'force recreating configuration also if up to date',
  type: 'boolean'
}).option('verbose', {
  alias: 'v',
  describe: 'run in verbose mode (multiple makes more verbose)',
  count: true,
  global: true
}).option('quiet', {
  alias: 'q',
  describe: 'don\'t output header and footer',
  type: 'boolean',
  global: true
}).option('nocolors', {
  alias: 'C',
  describe: 'turn of color output',
  type: 'boolean',
  global: true
}).option('help', {
  alias: 'h',
  description: 'display help message'
});

_yargs2.default.help('help').epilogue('You may use environment variables prefixed with \'VALIDATOR_\' to set any of \
the options like \'VALIDATOR_VERBOSE\' to set the verbose level.\n\
For more information, look into the man page.').completion('bashrc-script').strict().fail(err => {
  err = new Error(err);
  err.description = 'Specify --help for usage';
  _alinexCore2.default.exit(2, err);
});

const argv = _yargs2.default.argv;

if (argv.output) {
  new _index2.default().transform(argv.input, argv.schema, argv.output, argv).catch(err => {
    console.log(err.message);
  });
} else new _index2.default().check(argv.input, argv.schema);