'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _es6Promisify = require('es6-promisify');

var _es6Promisify2 = _interopRequireDefault(_es6Promisify);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _alinexUtil = require('alinex-util');

var _alinexUtil2 = _interopRequireDefault(_alinexUtil);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = (0, _debug2.default)('validator');

function resolveSearch(data, search) {
  const dataList = [];
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = (typeof data === 'string' ? [data] : data)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      const e = _step.value;
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = search[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          const t = _step2.value;
          dataList.push(`${t}${e}`);
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return dataList;
}

function sharedStart(array) {
  const A = array.concat().sort();
  const a1 = A[0];
  const a2 = A[A.length - 1];
  const L = a1.length;
  let i = 0;
  while (i < L && a1.charAt(i) === a2.charAt(i)) i += 1;
  return a1.substring(0, i);
}

const write = (data, file) => {
  const writer = (0, _es6Promisify2.default)(_fs2.default.writeFile);
  return writer(file, JSON.stringify(data));
};

const schema = def => {
  if (typeof def === 'string') return Promise.resolve().then(() => require(`${def}`));
  return Promise.resolve(def);
};

class Validator {

  constructor() {
    this.search = [''];
  }

  searchApp(name, sub = 'config') {
    const gpath = sub === 'config' ? `/etc/${name}/` : `/var/lib/${name}/${sub}/`;
    const upath = `${_os2.default.homedir()}/.${name}/${sub}/`;
    this.search.push(gpath);
    this.search.push(upath);
    debug(`Search path for ${sub} is set to ${gpath} and ${upath}`);
    return this;
  }

  schema(def) {
    return schema(def);
  }

  load(data) {
    debug(`search for data files at ${_alinexUtil2.default.inspect(data)}`);

    const dataList = resolveSearch(data, this.search);

    return Promise.resolve().then(() => require('alinex-format')).then(format => {
      const reader = (0, _es6Promisify2.default)(_fs2.default.readFile);
      const parser = (0, _es6Promisify2.default)(format.parse);
      const searchGlob = (0, _es6Promisify2.default)(_glob2.default);
      return Promise.all(dataList.map(e => searchGlob(e))).then(res => res.reduce((acc, val) => acc.concat(val), [])).then(files => {
        var _iteratorNormalCompletion3 = true;
        var _didIteratorError3 = false;
        var _iteratorError3 = undefined;

        try {
          for (var _iterator3 = files[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            const f = _step3.value;
            debug(`found data file at ${f}`);
          }
        } catch (err) {
          _didIteratorError3 = true;
          _iteratorError3 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion3 && _iterator3.return) {
              _iterator3.return();
            }
          } finally {
            if (_didIteratorError3) {
              throw _iteratorError3;
            }
          }
        }

        return files;
      }).then(files => {
        const base = sharedStart(files.map(e => _path2.default.dirname(e)));
        return Promise.all(files.map(file => reader(file).then(content => parser(content)).then(content => {
          const dir = _path2.default.dirname(file).substr(base.length + 1);
          if (dir) {
            var _iteratorNormalCompletion4 = true;
            var _didIteratorError4 = false;
            var _iteratorError4 = undefined;

            try {
              for (var _iterator4 = dir.split('/')[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
                const e = _step4.value;

                const obj = {};
                obj[e] = content;
                content = obj;
              }
            } catch (err) {
              _didIteratorError4 = true;
              _iteratorError4 = err;
            } finally {
              try {
                if (!_iteratorNormalCompletion4 && _iterator4.return) {
                  _iterator4.return();
                }
              } finally {
                if (_didIteratorError4) {
                  throw _iteratorError4;
                }
              }
            }
          }
          return content;
        })));
      }).then(list => list.reduce((acc, val) => _alinexUtil2.default.extend(acc, val), {}));
    });
  }

  check(data, def) {
    const list = [];

    if (def instanceof Promise) list.push(def);else list.push(schema(def));
    if (data instanceof Promise) list.push(data);else list.push(this.load(data));

    return Promise.all(list).then(values => values[0].validate(values[1], typeof data === 'string' ? data : undefined));
  }

  transform(data, def, file, opt) {
    const stat = (0, _es6Promisify2.default)(_fs2.default.stat);
    let p = Promise.resolve();
    if (!(opt && opt.force)) {
      const dataList = resolveSearch(data, this.search);

      const list = [def, file].map(e => stat(e).catch(() => false));

      list.unshift(Promise.all(dataList.map(e => stat(e).catch(() => false))).then(res => res.reduce((acc, val) => !acc || val.mtime > acc ? val : acc, false)));
      p = p.then(() => Promise.all(list)).then(res => {
        if (!res[0] || !res[2] || res[0].mtime > res[2].mtime) return Promise.resolve();
        if (def instanceof Promise || !res[1] || res[1].mtime > res[2].mtime) return Promise.resolve();
        return Promise.reject();
      });
    }

    return p.then(() => this.check(data, def)).then(result => write(result, file).then(() => result)).catch(() => {
      debug('JSON file found and newer than data and definition');
      return Promise.reject(new Error('No need to create configuration again because it\'s up to date.'));
    });
  }
}

exports.default = Validator;