'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _es6Promisify = require('es6-promisify');

var _es6Promisify2 = _interopRequireDefault(_es6Promisify);

var _punycode = require('punycode');

var _punycode2 = _interopRequireDefault(_punycode);

var _String = require('./String');

var _String2 = _interopRequireDefault(_String);

var _Error = require('../Error');

var _Error2 = _interopRequireDefault(_Error);

var _Reference = require('../Reference');

var _Reference2 = _interopRequireDefault(_Reference);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class DomainSchema extends _String2.default {
  constructor(base) {
    super(base);

    let raw = this._rules.descriptor.pop();
    this._rules.descriptor.push(this._dnsDescriptor, this._formatDescriptor, raw);
    raw = this._rules.validator.pop();
    this._rules.validator.push(this._dnsValidator, this._formatValidator, raw);
    super.stripEmpty();
  }

  stripEmpty() {
    return this._setError('stripEmpty');
  }
  truncate() {
    return this._setError('truncate');
  }
  pad() {
    return this._setError('pad');
  }

  _typeDescriptor() {
    return 'It has to be a reasonable hostname.\n';
  }

  _typeValidator(data) {
    if (typeof data.value !== 'string') {
      return Promise.reject(new _Error2.default(this, data, 'A text string is needed.'));
    }

    data.value = _punycode2.default.toASCII(data.value);
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = data.value.split('.')[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        const p = _step.value;

        if (p.length > 63) {
          return Promise.reject(new _Error2.default(this, data, `Each label within the domain name \
should be smaller than 64 characters: '${p}' is to large`));
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    if (data.value.length > 253) {
      return Promise.reject(new _Error2.default(this, data, 'The full domain name is too large, such \
names are not allowed in DNS'));
    }
    if (!data.value.match(/^[a-zA-Z0-9][a-zA-Z0-9-]{1,61}[a-zA-Z0-9](?:\.[a-zA-Z]{2,})+$/)) {
      return Promise.reject(new _Error2.default(this, data, 'Invalid domain name, only a-z, 0-9 and \
`-` is allowed'));
    }
    return Promise.resolve();
  }

  _allowValidator(data) {
    const check = this._check;
    this._checkArrayString('allow');
    this._checkArrayString('deny');

    let denyPriority = 0;
    let allowPriority = 0;
    if (check.deny && check.deny.length) {
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = check.deny[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          const e = _step2.value;

          const domain = _punycode2.default.toASCII(e);
          if (data.value === domain) {
            denyPriority = 99;
            break;
          }
          if (data.value.endsWith(`.${domain}`)) {
            const m = domain.match(/\./g);
            const level = m ? m.length + 1 : 1;
            if (level > denyPriority) denyPriority = level;
          }
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    }
    if (check.allow && check.allow.length) {
      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = check.allow[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          const e = _step3.value;

          const domain = _punycode2.default.toASCII(e);
          if (data.value === domain) {
            allowPriority = 99;
            break;
          }
          if (data.value.endsWith(`.${domain}`)) {
            const m = domain.match(/\./g);
            const level = m ? m.length + 1 : 1;
            if (level > allowPriority) allowPriority = level;
          }
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }
    }
    if (denyPriority > allowPriority) {
      return Promise.reject(new _Error2.default(this, data, 'Domain found in blacklist (denied item).'));
    }
    return Promise.resolve();
  }

  _lengthDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.min instanceof _Reference2.default) {
      msg += `Minimum number of labels depends on ${set.min.description}. `;
    }
    if (set.max instanceof _Reference2.default) {
      msg += `Maximum number of labels depends on ${set.max.description}. `;
    }
    if (!this._isReference('min') && !this._isReference('max') && set.min && set.max) {
      msg = set.min === set.max ? `The string has to contain exactly ${set.min} labels. ` : `The string can have between ${set.min} and ${set.max} labels. `;
    } else if (!this._isReference('min') && set.min) {
      msg = `The string needs at least ${set.min} labels. `;
    } else if (!this._isReference('max') && set.max) {
      msg = `The string allows up to ${set.min} labels. `;
    }
    return msg.length ? `${msg.trim()}\n` : msg;
  }

  _lengthValidator(data) {
    const check = this._check;
    try {
      this._checkNumber('min');
      this._checkNumber('max');
      if (check.max && check.min && check.min > check.max) {
        throw new Error('Min label number can´t be greater than max label number');
      }
    } catch (err) {
      return Promise.reject(new _Error2.default(this, data, err.message));
    }

    const num = data.value.split('.').length;
    if (check.min && num < check.min) {
      return Promise.reject(new _Error2.default(this, data, `The string has ${num} labels. \
 This is too less, at least ${check.min} labesl are needed.`));
    }
    if (check.max && num > check.max) {
      return Promise.reject(new _Error2.default(this, data, `The string has ${num} labels. \
 This is too much, not more than ${check.max} labels are allowed.`));
    }
    return Promise.resolve();
  }

  dns(type) {
    const set = this._setting;
    if (type === false) delete set.dns;else if (typeof type === 'string') set.dns = [type];else set.dns = type || true;
    return this;
  }

  _dnsDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.dns) {
      if (this._isReference('dns')) {
        msg += `The domain should has a valid record if set under \
${set.dns.description}. `;
      } else if (set.dns === true) msg += 'The domain should has at least one DNS entry. ';else msg += `The domain should has at least one DNS record of type ${set.dns.join(', ')}. `;
    }
    return msg.length ? `${msg.trim()}\n` : msg;
  }

  _dnsValidator(data) {
    const check = this._check;
    if (check.dns) {
      if (check.dns === true) check.dns = ['ANY'];else if (typeof check.dns === 'string') check.dns = [check.dns];else if (!Array.isArray(check.dns)) {
        return Promise.reject(new _Error2.default(this, data, 'The dns setting is no string or string array'));
      }
    }

    if (check.dns) {
      return Promise.resolve().then(() => require('dns')).then(dns => {
        const resolve = (0, _es6Promisify2.default)(dns.resolve);
        const p = check.dns.map(e => {
          const r = e === 'ANY' ? resolve(data.value) : resolve(data.value, e);
          return r.catch(() => undefined);
        });
        return Promise.all(p).then(list => {
          const success = list.filter(e => Array.isArray(e) && e.length).length;
          if (!success) {
            return Promise.reject(new _Error2.default(this, data, `No DNS record of type \
${check.dns.join(', ')} found`));
          }
          return Promise.resolve();
        });
      });
    }
    return Promise.resolve();
  }

  punycode(flag) {
    return this._setFlag('punycode', flag);
  }
  resolve(flag) {
    return this._setFlag('resolve', flag);
  }

  _formatDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.punycode) {
      if (this._isReference('punycode')) {
        msg += `The domain is converted into its ASCII presentation if defined under \
${set.punycode.description}. `;
      } else msg += 'The domain is converted to its ASCII presentation. ';
    }
    if (set.resolve) {
      if (this._isReference('resolve')) {
        msg += `The domain is resolved into its IP address if defined under \
${set.resolve.description}. `;
      } else msg += 'The domain is resolved into its IP address. ';
    }
    return msg.length ? `${msg.trim()}\n` : msg;
  }

  _formatValidator(data) {
    const check = this._check;
    try {
      this._checkBoolean('punycode');
      this._checkBoolean('resolve');
    } catch (err) {
      return Promise.reject(new _Error2.default(this, data, err.message));
    }

    if (!check.punycode) data.value = _punycode2.default.toUnicode(data.value);
    if (check.resolve) {
      return Promise.resolve().then(() => require('dns')).then(dns => (0, _es6Promisify2.default)(dns.lookup)(data.value).then(resolved => {
        data.value = resolved;
        return true;
      }));
    }
    return Promise.resolve();
  }
}

exports.default = DomainSchema;