'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

var _Schema = require('./Schema');

var _Schema2 = _interopRequireDefault(_Schema);

var _Error = require('../Error');

var _Error2 = _interopRequireDefault(_Error);

var _Data = require('../Data');

var _Data2 = _interopRequireDefault(_Data);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class LogicSchema extends _Schema2.default {
  constructor(base) {
    super(base);

    let raw = this._rules.descriptor.pop();

    this._rules.descriptor.push(this._logicDescriptor, raw);
    raw = this._rules.validator.pop();
    this._rules.validator.push(this._logicValidator, raw);
  }

  required() {
    return this._setError('required');
  }
  forbidden() {
    return this._setError('forbidden');
  }

  inspect(depth, options) {
    const newOptions = Object.assign({}, options, {
      depth: options.depth === null ? null : options.depth - 1
    });
    const padding = ' '.repeat(5);
    const inner = _util2.default.inspect(this._setting.logic, newOptions).replace(/\n/g, `\n${padding}`);
    return `${options.stylize(this.constructor.name, 'class')} ${inner} `;
  }

  if(schema) {
    const set = this._setting;
    if (set.logic) throw new Error(`Logic is already started using ${set.logic[0][0]}`);
    set.logic = [];
    set.logic.push(['if', schema]);
    return this;
  }

  then(schema) {
    const set = this._setting;
    if (!set.logic) throw new Error('Logic is not defined use `if()` first');
    if (set.logic[0][0] !== 'if') {
      throw new Error('Logic is not defined as `if()`, only `and` and `or` are allowed here');
    }
    if (set.logic[1]) throw new Error('Then condition in logic is already set');
    set.logic[1] = ['then', schema];
    return this;
  }

  else(schema) {
    const set = this._setting;
    if (!set.logic) throw new Error('Logic is not defined use `if()` first');
    if (set.logic[0][0] !== 'if') {
      throw new Error('Logic is not defined as `if()`, only `and` and `or` are allowed here');
    }
    if (set.logic[2]) throw new Error('Then condition in logic is already set');
    set.logic[2] = ['else', schema];
    return this;
  }

  allow(schema) {
    const set = this._setting;
    if (set.logic) throw new Error(`Logic is already started using ${set.logic[0][0]}`);
    set.logic = [];
    set.logic.push(['allow', schema]);
    return this;
  }

  deny(schema) {
    const set = this._setting;
    if (set.logic) throw new Error(`Logic is already started using ${set.logic[0][0]}`);
    set.logic = [];
    set.logic.push(['deny', schema]);
    return this;
  }

  and(schema) {
    const set = this._setting;
    if (!set.logic) throw new Error('Logic is not defined use `allow()` or `deny()` first');
    set.logic.push(['and', schema]);
    return this;
  }

  or(schema) {
    const set = this._setting;
    if (!set.logic) throw new Error('Logic is not defined use `allow()` or `deny()` first');
    set.logic.push(['or', schema]);
    return this;
  }

  _logicDescriptor() {
    const set = this._setting;
    if (!set.logic) return '';
    const msg = 'The validation depends on the following logic:\n';
    return msg + set.logic.map(e => `- **${e[0].toUpperCase()}**: ${e[1].description.replace(/\n/g, '\n  ')}\n`).join('');
  }

  _logicValidator(data) {
    const check = this._check;
    if (!check.logic) return Promise.resolve();
    let logic = check.logic.slice(0);
    if (logic[0][0] === 'if') {
      return logic[0][1]._validate(data.clone).then(() => logic[1] ? logic[1][1]._validate(data) : data, () => logic[2] ? logic[2][1]._validate(data) : data);
    }

    let p = Promise.resolve();
    logic.forEach((v, i) => {
      var _v = _slicedToArray(v, 2);

      const op = _v[0],
            schema = _v[1];

      if (op === 'and') {
        p = p.then(last => {
          if (last instanceof _Data2.default) return schema._validate(last.clone);
          return undefined;
        }).then(last => {
          logic[i][1] = last;
          return last;
        }).catch(err => {
          logic[i][1] = err;
          return undefined;
        });
      } else {
        p = p.then(() => schema._validate(data.clone)).then(last => {
          logic[i][1] = last;
          return last;
        }).catch(err => {
          logic[i][1] = err;
          return undefined;
        });
      }
    });

    p = p.then(() => {
      let last = 0;
      logic.forEach((v, i) => {
        var _v2 = _slicedToArray(v, 2);

        const op = _v2[0],
              res = _v2[1];

        if (op === 'and') {
          if (res instanceof _Data2.default) logic[last][1] = res;else if (logic[last][1] instanceof _Data2.default) logic[last][1] = res;
          delete logic[i];
        } else last = i;
      });
      logic = logic.filter(e => e);
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = logic[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          const v = _step.value;

          if (v[1] instanceof _Data2.default) {
            logic[0][1] = v[1];
            break;
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      if (logic[0][0] === 'allow') {
        if (logic[0][1] instanceof _Data2.default) data.value = logic[0][1].value;else throw logic[0][1];
      } else if (logic[0][1] instanceof _Data2.default) {
        throw new _Error2.default(this, data, 'The element is denied by logic');
      }
    });

    return p;
  }
}

exports.default = LogicSchema;