'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

var _Schema = require('./Schema');

var _Schema2 = _interopRequireDefault(_Schema);

var _Error = require('../Error');

var _Error2 = _interopRequireDefault(_Error);

var _Reference = require('../Reference');

var _Reference2 = _interopRequireDefault(_Reference);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Logic {

  constructor(type, key, peers) {
    this.type = type;
    if (key) this.key = key;
    this.peers = peers;
  }
}

class ObjectSchema extends _Schema2.default {
  constructor(base) {
    super(base);

    let raw = this._rules.descriptor.pop();
    this._rules.descriptor.push(this._typeDescriptor, this._structureDescriptor, this._copyDescriptor, this._keysDescriptor, this._removeDescriptor, this._requiredKeysDescriptor, this._logicDescriptor, this._lengthDescriptor, raw);
    raw = this._rules.validator.pop();
    this._rules.validator.push(this._typeValidator, this._structureValidator, this._copyValidator, this._keysValidator, this._removeValidator, this._requiredKeysValidator, this._logicValidator, this._lengthValidator, raw);
  }

  _typeDescriptor() {
    return 'A data object is needed.\n';
  }

  _typeValidator(data) {
    if (typeof data.value !== 'object') {
      return Promise.reject(new _Error2.default(this, data, 'A data object is needed.'));
    }
    return Promise.resolve();
  }

  deepen(value) {
    return this._setAny('deepen', value);
  }
  flatten(value) {
    return this._setAny('flatten', value);
  }

  _structureDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.deepen instanceof _Reference2.default) {
      msg += `Key names will be split on ${set.deepen.description}. `;
    } else if (set.deepen) {
      msg += `Key names will be split on \
 \`${typeof set.deepen === 'string' ? set.deepen : _util2.default.inspect(set.deepen)}\` \
 into deeper structures. `;
    }
    if (set.flatten instanceof _Reference2.default) {
      msg += `Deep structures will be flattened depending on ${set.flatten.description} \
as separator. `;
    } else if (set.flatten) {
      msg += `Deep structures will be flattened by combining key names using \
 \`${set.flatten}\` as separator. `;
    }
    return msg.length ? `${msg.trim()}\n` : msg;
  }

  _structureValidator(data) {
    const check = this._check;
    try {
      this._checkMatch('deepen');
      this._checkString('flatten');
    } catch (err) {
      return Promise.reject(new _Error2.default(this, data, err.message));
    }

    if (check.deepen) {
      Object.keys(data.value).forEach(key => {
        const value = data.value[key];
        const parts = key.split(check.deepen);
        if (parts.length > 1) {
          let obj = data.value;
          const last = parts.pop();
          parts.forEach(e => {
            if (!obj[e]) obj[e] = {};
            obj = obj[e];
          });
          obj[last] = value;
          delete data.value[key];
        }
      });
    }
    if (check.flatten) {
      const result = {};
      const separator = check.flatten;
      function recurse(cur, prop) {
        if (Object(cur) !== cur) {
          result[prop] = cur;
        } else if (Array.isArray(cur) && cur.length) {
          for (let i = 0, l = cur.length; i < l; i += 1) recurse(cur[i], `${prop}${separator}${i}`);
        } else if (Object.keys(cur).length) {
          for (const p in cur) {
            if (Object.prototype.hasOwnProperty.call(cur, p)) {
              recurse(cur[p], prop ? `${prop}${separator}${p}` : p);
            }
          }
        } else {
          result[prop] = cur;
        }
      }
      recurse(data.value, '');
      data.value = result;
    }
    return Promise.resolve();
  }

  copy(from, to, force = false) {
    const set = this._setting;
    if (from === undefined && to) throw new Error('Always two key names are required for copy');
    if (to === undefined && from) throw new Error('Always two key names are required for copy');
    if (from === undefined) delete set.copy;else {
      if (!set.copy) set.copy = [];
      set.copy.push([from, to, false, force]);
    }
    return this;
  }

  move(from, to, force = false) {
    const set = this._setting;
    if (from === undefined && to) throw new Error('Always two key names are required for move');
    if (to === undefined && from) throw new Error('Always two key names are required for move');
    if (from === undefined) delete set.copy;else {
      if (!set.copy) set.copy = [];
      set.copy.push([from, to, true, force]);
    }
    return this;
  }

  _copyDescriptor() {
    const set = this._setting;
    let msg = '';

    if (set.copy) {
      msg += 'It will be copied/moved to:\n';
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = set.copy[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          const rule = _step.value;

          msg += `- ${rule[2] ? 'move' : 'copy'} \
${rule[0] instanceof _Reference2.default ? rule[0].description : rule[0]} to \
${rule[1] instanceof _Reference2.default ? rule[1].description : rule[1]} \
${rule[3] ? '(overwrite existing)' : '(if not exists)'}\n`;
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }
    return `${msg}\n`;
  }

  _copyValidator(data) {
    const check = this._check;
    try {
      if (check.copy) {
        check.copy.forEach((e, i) => {
          if (typeof e[0] !== 'string') {
            throw new Error(`No string value for from \`key\` setting given in \
      ${this._setting.copy[i] && this._setting.copy[i].description || this._setting.copy[i]}`);
          }
          if (typeof e[1] !== 'string') {
            throw new Error(`No string value for to \`key\` setting given in \
      ${this._setting.copy[i] && this._setting.copy[i].description || this._setting.copy[i]}`);
          }
        });
      }
    } catch (err) {
      return Promise.reject(new _Error2.default(this, data, err.message));
    }

    if (check.copy) {
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = check.copy[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          const rule = _step2.value;

          if (rule[3] || !data.value[rule[1]]) {
            data.value[rule[1]] = data.value[rule[0]];
            if (rule[2]) delete data.value[rule[0]];
          }
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    }
    return Promise.resolve();
  }

  key(name, check) {
    const set = this._setting;
    if (name === undefined) delete set.keys;else {
      if (!set.keys) set.keys = new Map();
      if (check === undefined) set.keys.delete(name);else set.keys.set(name, check);
    }
    return this;
  }

  _keysDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.keys) {
      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = set.keys[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          const _ref = _step3.value;

          var _ref2 = _slicedToArray(_ref, 2);

          const key = _ref2[0];
          const schema = _ref2[1];

          msg += `- \`${typeof key === 'string' ? key : _util2.default.inspect(key)}\`: \
${schema.description.replace(/\n/g, '\n  ')}\n`;
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      if (msg.length) msg = `The following keys have a special format:\n${msg}\n`;
    }
    return msg;
  }

  _keysValidator(data) {
    const check = this._check;
    try {
      this._checkObject('keys');
    } catch (err) {
      return Promise.reject(new _Error2.default(this, data, err.message));
    }

    const checks = [];
    const keys = [];
    const sum = {};
    Object.keys(data.value).forEach(key => {
      let found = false;
      let schema = check.keys[key];
      if (schema) {
        checks.push(schema._validate(data.sub(key)));
        keys.push(key);
        found = true;
      } else if (Object.keys(check.keys).length > 0) {
        var _iteratorNormalCompletion4 = true;
        var _didIteratorError4 = false;
        var _iteratorError4 = undefined;

        try {
          for (var _iterator4 = Object.keys(check.keys)[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
            const k = _step4.value;

            if (k.match(/^\/([^\\/]|\\.)+\/[gi]*$/)) {
              const parts = k.match(/([^\\/]|\\.)+/g);
              const re = new RegExp(parts[0], parts[1]);
              schema = check.keys[k];
              if (key.match(re)) {
                checks.push(schema._validate(data.sub(key)));
                keys.push(key);
                found = true;
                break;
              }
            }
          }
        } catch (err) {
          _didIteratorError4 = true;
          _iteratorError4 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion4 && _iterator4.return) {
              _iterator4.return();
            }
          } finally {
            if (_didIteratorError4) {
              throw _iteratorError4;
            }
          }
        }
      }

      if (!found) {
        if (!data.temp.unchecked) data.temp.unchecked = [];
        data.temp.unchecked[key] = true;
        sum[key] = data.value[key];
      }
    });

    return Promise.all(checks).then(result => {
      if (result) {
        result.forEach((e, i) => {
          sum[keys[i]] = e.value;
        });
      }
      data.value = sum;
      return Promise.resolve();
    });
  }

  removeUnknown(flag) {
    return this._setFlag('removeUnknown', flag);
  }
  denyUnknown(flag) {
    return this._setFlag('denyUnknown', flag);
  }

  _removeDescriptor() {
    const set = this._setting;
    if (set.removeUnknown instanceof _Reference2.default) {
      return `Keys not defined with the rules before will be removed depending on \
${set.removeUnknown.description}.\n`;
    }
    if (set.removeUnknown) return 'Keys not defined with the rules before will be removed.\n';
    return '';
  }

  _removeValidator(data) {
    const check = this._check;
    try {
      this._checkBoolean('removeUnknown');
      this._checkBoolean('denyUnknown');
    } catch (err) {
      return Promise.reject(new _Error2.default(this, data, err.message));
    }

    if (check.removeUnknown) {
      for (const key in data.temp.unchecked) if (key) delete data.value[key];
    }
    if (check.denyUnknown) {
      for (const key in data.temp.unchecked) {
        if (key) {
          return Promise.reject(new _Error2.default(this, data, `The key ${key} is not defined and therefore denied here.`));
        }
      }
    }
    return Promise.resolve();
  }

  min(value) {
    const set = this._setting;
    if (value) {
      if (!(value instanceof _Reference2.default)) {
        const int = parseInt(value, 10);
        if (int < 0) throw new Error('Length for min() has to be positive');
        if (set.max && !this._isReference('max') && value > set.max) {
          throw new Error('Length for min() should be equal or below max');
        }
      }
      set.min = value;
    } else delete set.min;
    return this;
  }

  max(value) {
    const set = this._setting;
    if (value) {
      if (!(value instanceof _Reference2.default)) {
        const int = parseInt(value, 10);
        if (int <= 0) throw new Error('Length for max() has to be positive');
        if (set.min && !this._isReference('min') && value < set.min) {
          throw new Error('Length for max() should be equal or above min');
        }
      }
      set.max = value;
    } else delete set.max;
    return this;
  }

  length(value) {
    const set = this._setting;
    if (value) {
      if (!(value instanceof _Reference2.default)) {
        const int = parseInt(value, 10);
        if (int <= 0) throw new Error('Length has to be positive');
      }
      set.min = value;
      set.max = value;
    } else {
      delete set.min;
      delete set.max;
    }
    return this;
  }

  _lengthDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.min || set.max) {
      if (set.min === set.max) {
        if (this._isReference('min')) {
          return `The object has to contain the number of elements specified in \
${set.min.description}.\n`;
        }
        return `The object has to contain exactly ${set.min} elements.\n`;
      }
      if (this._isReference('min')) {
        msg += `The object needs at have at least the specified in ${set.min.description} \
number of elements. `;
      } else {
        msg += `The object needs at least ${set.min} elements. `;
      }
      if (this._isReference('max')) {
        msg += `The object can't have more than specified in ${set.max.description} \
elements. `;
      } else {
        msg += `The object allows up to ${set.min} elements. `;
      }
      if (set.min && set.max && !this._isReference('min') && !this._isReference('max')) {
        return `The object needs between ${set.min} and ${set.max} elements.\n`;
      }
    }
    return msg.length ? msg.replace(/ $/, '\n') : msg;
  }

  _lengthValidator(data) {
    const check = this._check;
    try {
      this._checkNumber('min');
      this._checkNumber('max');
    } catch (err) {
      return Promise.reject(new _Error2.default(this, data, err.message));
    }

    const num = Object.keys(data.value).length;
    if (check.min && num < check.min) {
      return Promise.reject(new _Error2.default(this, data, `The object should has a length of ${num} elements. \
This is too less, at least ${check.min} are needed.`));
    }
    if (check.max && num > check.max) {
      return Promise.reject(new _Error2.default(this, data, `The object should has a length of ${num} elements. \
This is too much, not more than ${check.max} are allowed.`));
    }
    return Promise.resolve();
  }

  requiredKeys(...values) {
    const set = this._setting;
    const value = values.reduce((acc, val) => acc.concat(val), []);
    if (value.length === 1 && value[0] === undefined) delete set.requiredKeys;else if (value.length === 1 && value[0] instanceof _Reference2.default) set.requiredKeys = value[0];else {
      if (!set.requiredKeys) set.requiredKeys = new Set();
      var _iteratorNormalCompletion5 = true;
      var _didIteratorError5 = false;
      var _iteratorError5 = undefined;

      try {
        for (var _iterator5 = value[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
          const e = _step5.value;

          if (value === undefined) set.required = false;
          set.requiredKeys.add(e);
          if (set.forbiddenKeys) set.forbiddenKeys.delete(e);
        }
      } catch (err) {
        _didIteratorError5 = true;
        _iteratorError5 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion5 && _iterator5.return) {
            _iterator5.return();
          }
        } finally {
          if (_didIteratorError5) {
            throw _iteratorError5;
          }
        }
      }
    }
    return this;
  }
  forbiddenKeys(...values) {
    const set = this._setting;
    const value = values.reduce((acc, val) => acc.concat(val), []);
    if (value.length === 1 && value[0] === undefined) delete set.forbiddenKeys;else if (value.length === 1 && value[0] instanceof _Reference2.default) set.forbiddenKeys = value[0];else {
      if (!set.forbiddenKeys) set.forbiddenKeys = new Set();
      var _iteratorNormalCompletion6 = true;
      var _didIteratorError6 = false;
      var _iteratorError6 = undefined;

      try {
        for (var _iterator6 = value[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
          const e = _step6.value;

          if (value === undefined) set.required = true;
          set.forbiddenKeys.add(e);
          if (set.requiredKeys) set.requiredKeys.delete(e);
        }
      } catch (err) {
        _didIteratorError6 = true;
        _iteratorError6 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion6 && _iterator6.return) {
            _iterator6.return();
          }
        } finally {
          if (_didIteratorError6) {
            throw _iteratorError6;
          }
        }
      }
    }
    return this;
  }

  _requiredKeysDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.forbiddenKeys instanceof _Reference2.default) {
      msg += `The keys within ${set.forbiddenKeys.description} are not allowed. `;
    } else if (set.forbiddenKeys && set.forbiddenKeys.size) {
      msg += `The keys ${Array.from(set.forbiddenKeys).join(', ').replace(/(.*),/, '$1 and')} \
are not allowed. `;
    }
    if (set.requiredKeys instanceof _Reference2.default) {
      msg += `Only the keys within ${set.requiredKeys.description} are required. `;
    } else if (set.requiredKeys && set.requiredKeys.size) {
      msg += `Only the keys ${Array.from(set.requiredKeys).join(', ').replace(/(.*),/, '$1 and')} \
are allowed. `;
    }
    return msg.length ? `${msg.trim()}\n` : '';
  }

  _requiredKeysValidator(data) {
    const check = this._check;
    const keys = Object.keys(data.value);
    this._checkArrayString('requiredKeys');
    this._checkArrayString('forbiddenKeys');

    if (check.forbiddenKeys && check.forbiddenKeys.length) {
      var _iteratorNormalCompletion7 = true;
      var _didIteratorError7 = false;
      var _iteratorError7 = undefined;

      try {
        for (var _iterator7 = check.forbiddenKeys[Symbol.iterator](), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
          const e = _step7.value;

          if (keys.includes(e)) {
            return Promise.reject(new _Error2.default(this, data, `The key ${e} is not allowed here. `));
          }
        }
      } catch (err) {
        _didIteratorError7 = true;
        _iteratorError7 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion7 && _iterator7.return) {
            _iterator7.return();
          }
        } finally {
          if (_didIteratorError7) {
            throw _iteratorError7;
          }
        }
      }
    }
    if (check.requiredKeys && check.requiredKeys.length) {
      var _iteratorNormalCompletion8 = true;
      var _didIteratorError8 = false;
      var _iteratorError8 = undefined;

      try {
        for (var _iterator8 = check.requiredKeys[Symbol.iterator](), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
          const e = _step8.value;

          if (!keys.includes(e)) {
            return Promise.reject(new _Error2.default(this, data, `The key ${e} is missing. `));
          }
        }
      } catch (err) {
        _didIteratorError8 = true;
        _iteratorError8 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion8 && _iterator8.return) {
            _iterator8.return();
          }
        } finally {
          if (_didIteratorError8) {
            throw _iteratorError8;
          }
        }
      }
    }
    return Promise.resolve();
  }

  and(...keys) {
    const set = this._setting;

    const list = keys.reduce((acc, val) => acc.concat(val), []);
    if (!set.logic) set.logic = [];
    set.logic.push(new Logic('and', undefined, list));
    return this;
  }

  nand(...keys) {
    const set = this._setting;

    const list = keys.reduce((acc, val) => acc.concat(val), []);
    if (!set.logic) set.logic = [];
    set.logic.push(new Logic('nand', undefined, list));
    return this;
  }

  or(...keys) {
    const set = this._setting;

    const list = keys.reduce((acc, val) => acc.concat(val), []);
    if (!set.logic) set.logic = [];
    set.logic.push(new Logic('or', undefined, list));
    return this;
  }

  xor(...keys) {
    const set = this._setting;

    const list = keys.reduce((acc, val) => acc.concat(val), []);
    if (!set.logic) set.logic = [];
    set.logic.push(new Logic('xor', undefined, list));
    return this;
  }

  with(key, ...peers) {
    const set = this._setting;

    const list = peers.reduce((acc, val) => acc.concat(val), []);
    if (!set.logic) set.logic = [];
    set.logic.push(new Logic('with', key, list));
    return this;
  }

  without(key, ...peers) {
    const set = this._setting;

    const list = peers.reduce((acc, val) => acc.concat(val), []);
    if (!set.logic) set.logic = [];
    set.logic.push(new Logic('without', key, list));
    return this;
  }

  clearLogic() {
    delete this._setting.logic;
    return this;
  }

  _logicDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.logic && set.logic.length) {
      var _iteratorNormalCompletion9 = true;
      var _didIteratorError9 = false;
      var _iteratorError9 = undefined;

      try {
        for (var _iterator9 = set.logic[Symbol.iterator](), _step9; !(_iteratorNormalCompletion9 = (_step9 = _iterator9.next()).done); _iteratorNormalCompletion9 = true) {
          const rule = _step9.value;

          if (rule.type === 'and') {
            const list = rule.peers.map(e => `\`${e}\``).join(', ').replace(/(.*),/, '$1 and');
            msg += `All or none of the keys ${list} have to be present. `;
          } else if (rule.type === 'nand') {
            const list = rule.peers.map(e => `\`${e}\``).join(', ').replace(/(.*),/, '$1 and');
            msg += `Some but not all of the keys ${list} can be present. `;
          } else if (rule.type === 'or') {
            const list = rule.peers.map(e => `\`${e}\``).join(', ').replace(/(.*),/, '$1 and');
            msg += `At least one of the keys ${list} have to be present. `;
          } else if (rule.type === 'xor') {
            const list = rule.peers.map(e => `\`${e}\``).join(', ').replace(/(.*),/, '$1 and');
            msg += `Exactly one of the keys ${list} have to be present. `;
          } else if (rule.type === 'with') {
            const list = rule.peers.map(e => `\`${e}\``).join(', ').replace(/(.*),/, '$1 and');
            msg += `If \`${rule.key}\` is set the keys ${list} have to be present, too. `;
          } else if (rule.type === 'without') {
            const list = rule.peers.map(e => `\`${e}\``).join(', ').replace(/(.*),/, '$1 and');
            msg += `If \`${rule.key}\` is set the keys ${list} are forbidden. `;
          }
        }
      } catch (err) {
        _didIteratorError9 = true;
        _iteratorError9 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion9 && _iterator9.return) {
            _iterator9.return();
          }
        } finally {
          if (_didIteratorError9) {
            throw _iteratorError9;
          }
        }
      }
    }
    return msg.length ? `${msg.trim()}\n` : msg;
  }

  _logicValidator(data) {
    const check = this._check;
    if (check.logic && check.logic.length) {
      const keys = Object.keys(data.value);
      var _iteratorNormalCompletion10 = true;
      var _didIteratorError10 = false;
      var _iteratorError10 = undefined;

      try {
        for (var _iterator10 = check.logic[Symbol.iterator](), _step10; !(_iteratorNormalCompletion10 = (_step10 = _iterator10.next()).done); _iteratorNormalCompletion10 = true) {
          const rule = _step10.value;

          if (rule.type === 'and') {
            const contained = rule.peers.filter(e => keys.includes(e));

            if (contained.length > 0 && contained.length !== rule.peers.length) {
              const list = rule.peers.map(e => `\`${e}\``).join(', ').replace(/(.*),/, '$1 and');
              return Promise.reject(new _Error2.default(this, data, `All or none of the keys ${list} have to be present \
 but there are only ${contained.length} of the ${rule.peers.length} keys present.`));
            }
          } else if (rule.type === 'nand') {
            const contained = rule.peers.filter(e => keys.includes(e));

            if (contained.length === rule.peers.length) {
              const list = rule.peers.map(e => `\`${e}\``).join(', ').replace(/(.*),/, '$1 and');
              return Promise.reject(new _Error2.default(this, data, `Some but not all of the keys ${list} can be present but all are set.`));
            }
          } else if (rule.type === 'or') {
            const contained = rule.peers.filter(e => keys.includes(e));

            if (!contained.length) {
              const list = rule.peers.map(e => `\`${e}\``).join(', ').replace(/(.*),/, '$1 and');
              return Promise.reject(new _Error2.default(this, data, `At least one of the keys ${list} have to be present but none are set.`));
            }
          } else if (rule.type === 'xor') {
            const contained = rule.peers.filter(e => keys.includes(e));

            if (contained.length !== 1) {
              const list = rule.peers.map(e => `\`${e}\``).join(', ').replace(/(.*),/, '$1 and');
              return Promise.reject(new _Error2.default(this, data, `Exactly one of the keys ${list} have to be present \
 but ${contained.length} are set.`));
            }
          } else if (rule.type === 'with') {
            const contained = rule.peers.filter(e => keys.includes(e));

            if (keys.includes(rule.key) && contained.length !== rule.peers.length) {
              const list = rule.peers.map(e => `\`${e}\``).join(', ').replace(/(.*),/, '$1 and');
              return Promise.reject(new _Error2.default(this, data, `If \`${rule.key}\` is set the keys ${list} have to be present \
 but there are only ${contained.length} of the ${rule.peers.length} keys present.`));
            }
          } else if (rule.type === 'without') {
            const contained = rule.peers.filter(e => keys.includes(e));

            if (keys.includes(rule.key) && contained.length) {
              const list = rule.peers.map(e => `\`${e}\``).join(', ').replace(/(.*),/, '$1 and');
              return Promise.reject(new _Error2.default(this, data, `If \`${rule.key}\` is set the keys ${list} are forbidden \
 but ${contained.length} keys are set.`));
            }
          }
        }
      } catch (err) {
        _didIteratorError10 = true;
        _iteratorError10 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion10 && _iterator10.return) {
            _iterator10.return();
          }
        } finally {
          if (_didIteratorError10) {
            throw _iteratorError10;
          }
        }
      }
    }
    return Promise.resolve();
  }
}

exports.default = ObjectSchema;