'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

var _Any = require('./Any');

var _Any2 = _interopRequireDefault(_Any);

var _Error = require('../Error');

var _Error2 = _interopRequireDefault(_Error);

var _Reference = require('../Reference');

var _Reference2 = _interopRequireDefault(_Reference);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let striptags;

class Pad {

  constructor(type = 'right', char = ' ') {
    this.type = type;
    this.char = char;
  }
}

class Replace {

  constructor(match, replace, name) {
    this.match = match;
    this.replace = replace;
    if (name) this.name = name;
  }
}

class StringSchema extends _Any2.default {
  constructor(base) {
    super(base);

    let raw = this._rules.descriptor.pop();
    let allow = this._rules.descriptor.pop();
    this._rules.descriptor.push(this._typeDescriptor, this._makeStringDescriptor, this._replaceDescriptor, this._caseDescriptor, this._checkDescriptor, this._lengthDescriptor, this._matchDescriptor, allow, raw);
    raw = this._rules.validator.pop();
    allow = this._rules.validator.pop();
    this._rules.validator.push(this._makeStringValidator, this._typeValidator, this._replaceValidator, this._caseValidator, this._checkValidator, this._lengthValidator, this._matchValidator, allow, raw);
  }

  _typeDescriptor() {
    return 'It has to be a text string.\n';
  }

  _typeValidator(data) {
    if (typeof data.value !== 'string') {
      return Promise.reject(new _Error2.default(this, data, 'A text string is needed.'));
    }
    return Promise.resolve();
  }

  makeString(flag) {
    return this._setFlag('makeString', flag);
  }

  _makeStringDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.makeString instanceof _Reference2.default) {
      msg += `It will be converted to string depending on ${set.makeString.description}. `;
    } else if (set.makeString) {
      msg += 'If the value is no string it will be converted to one. ';
    }
    return msg.replace(/ $/, '\n');
  }

  _makeStringValidator(data) {
    const check = this._check;
    try {
      this._checkBoolean('makeString');
    } catch (err) {
      return Promise.reject(new _Error2.default(this, data, err.message));
    }

    if (check.makeString && typeof data.value !== 'string') data.value = data.value.toString();
    if (typeof data.value !== 'string') {
      return Promise.reject(new _Error2.default(this, data, 'A `string` value is needed here.'));
    }
    return Promise.resolve();
  }

  trim(flag) {
    return this._setFlag('trim', flag);
  }

  replace(match, replace, name) {
    const set = this._setting;
    if (match === undefined) delete set.replace;else if (replace === undefined && typeof match === 'string') {
        const len = set.replace.length;
        set.replace = set.replace.filter(e => e.name !== match);
        if (len === set.replace.length) {
          throw new Error(`No replacer with the name \`${match}\` is defined`);
        }
      } else if (match && typeof match !== 'string') {
        if (!set.replace) set.replace = [];
        set.replace.push(new Replace(match, replace || '', name));
      } else throw new Error('Needs a RegExp as first argument to define `replace()`.');
    return this;
  }

  _replaceDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.trim instanceof _Reference2.default) {
      msg += `Whitespace character will be trimmed depending on ${set.trim.description}. `;
    } else if (set.trim) {
      msg += 'Whitespace characters at the begin and end of the string are removed. ';
    }
    if (set.replace && set.replace.length) {
      const list = set.replace.map(e => `- \`${_util2.default.inspect(e.match)}\` =>
        \`${e.replace}\`${e.name ? ` (${e.name})` : ''}`).join('\n');
      msg += `The following replacements will be done:\n${list}\n`;
    }
    return msg.length ? `${msg.replace(/ $/, '')}\n` : msg;
  }

  _replaceValidator(data) {
    const check = this._check;
    try {
      this._checkBoolean('trim');
    } catch (err) {
      return Promise.reject(new _Error2.default(this, data, err.message));
    }

    if (check.trim) data.value = data.value.trim();
    if (check.replace && check.replace.length) {
      check.replace.forEach(e => {
        data.value = data.value.replace(e.match, e.replace);
      });
    }
    return Promise.resolve();
  }

  upperCase(what = 'all') {
    const set = this._setting;
    if (what === false) delete set.upperCase;else {
      if (set.lowerCase === what) delete set.lowerCase;
      if (set.upperCase === true) set.upperCase = 'all';else set.upperCase = what;
    }
    return this;
  }

  lowerCase(what = 'all') {
    const set = this._setting;
    if (what === false) delete set.lowerCase;else {
      if (set.upperCase === what) delete set.upperCase;
      if (set.lowerCase === true) set.lowerCase = 'all';else set.lowerCase = what;
    }
    return this;
  }

  _caseDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.lowerCase === 'all') msg += 'Convert the whole text to lowerCase. ';else if (set.upperCase === 'all') msg += 'Convert the whole text to upperCase. ';
    if (set.lowerCase instanceof _Reference2.default) {
      msg += `Lower case is used depending on ${set.lowerCase.description}. `;
    } else if (set.upperCase instanceof _Reference2.default) {
      msg += `Upper case is used depending on ${set.upperCase.description}. `;
    }
    if (set.lowerCase === 'first') msg += 'Convert only the first letter to lowerCase. ';else if (set.upperCase === 'first') msg += 'Convert only the first letter to upperCase. ';
    return msg.length ? `${msg.replace(/ $/, '')}\n` : msg;
  }

  _caseValidator(data) {
    const check = this._check;
    try {
      this._checkBoolean('lowerCase');
      this._checkBoolean('upperCase');
    } catch (err) {}
    if (check.lowerCase === true) check.lowerCase = 'all';else if (check.lowerCase === false) delete check.lowerCase;
    if (check.upperCase === true) check.upperCase = 'all';else if (check.upperCase === false) delete check.upperCase;
    try {
      this._checkString('lowerCase');
      this._checkString('upperCase');
    } catch (err) {
      return Promise.reject(new _Error2.default(this, data, err.message));
    }

    if (check.lowerCase === 'all') data.value = data.value.toLowerCase();else if (check.upperCase === 'all') data.value = data.value.toUpperCase();
    if (check.lowerCase === 'first') {
      data.value = `${data.value.substr(0, 1).toLowerCase()}${data.value.substr(1)}`;
    } else if (check.upperCase === 'first') {
      data.value = `${data.value.substr(0, 1).toUpperCase()}${data.value.substr(1)}`;
    }
    return Promise.resolve();
  }

  alphaNum(flag) {
    return this._setFlag('alphaNum', flag);
  }
  hex(flag) {
    return this._setFlag('hex', flag);
  }
  controls(flag) {
    return this._setFlag('controls', flag);
  }
  noHTML(flag) {
    return this._setFlag('noHTML', flag);
  }
  stripDisallowed(flag) {
    return this._setFlag('stripDisallowed', flag);
  }

  _checkDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.alphaNum instanceof _Reference2.default) {
      msg += `Only alpha numerical characters are allowed depending on \
${set.alphaNum.description}. `;
    } else if (set.alphaNum) {
      msg += 'Only alpha numerical characters are allowed. ';
    } else if (set.hex instanceof _Reference2.default) {
      msg += `Only hexa decimal characters are allowed depending on \
${set.hex.description}. `;
    } else if (set.hex) {
      msg += 'Only hexa decimal characters are allowed. ';
    }
    if (set.controls instanceof _Reference2.default) {
      msg += `Control characters are allowed depending on ${set.controls.description}. `;
    } else if (set.controls) {
      msg += 'Control characters are allowed. ';
    }
    if (set.noHTML instanceof _Reference2.default) {
      msg += `No HTML tags are allowed depending on ${set.noHTML.description}. `;
    } else if (set.noHTML) {
      msg += 'No HTML tags are allowed. ';
    }
    if (set.stripDisallowed instanceof _Reference2.default) {
      msg += `All not allowed characters will be removed depending on \
${set.stripDisallowed.description}. `;
    } else if (set.stripDisallowed) {
      msg += 'All not allowed characters will be removed. ';
    }
    return msg.length ? `${msg.replace(/ $/, '')}\n` : msg;
  }

  _checkValidator(data) {
    const check = this._check;
    try {
      this._checkBoolean('stripDisallowed');
      this._checkBoolean('alphaNum');
      this._checkBoolean('hex');
      this._checkBoolean('controls');
      this._checkBoolean('noHTML');
    } catch (err) {
      return Promise.reject(new _Error2.default(this, data, err.message));
    }

    if (check.stripDisallowed) {
      if (check.alphaNum) data.value = data.value.replace(/\W/g, '');else if (check.hex) data.value = data.value.replace(/[^a-fA-F0-9]/g, '');
      if (!check.controls) data.value = data.value.replace(/[^\x20-\x7E]/g, '');
      if (check.noHTML) {
        if (!striptags) striptags = require('striptags');
        data.value = striptags(data.value);
      }
    } else {
      if (check.alphaNum && data.value.match(/\W/)) {
        return Promise.reject(new _Error2.default(this, data, 'Only alpha numerical characters (a-z, A-Z, 0-9 and _) are allowed.'));
      } else if (check.hex && data.value.match(/[^a-fA-F0-9]/)) {
        return Promise.reject(new _Error2.default(this, data, 'Only hexa decimal characters (a-f, A-F and 0-9) are allowed.'));
      }
      if (!check.controls && data.value.match(/[^\x20-\x7E]/)) {
        return Promise.reject(new _Error2.default(this, data, 'Control characters are not allowed.'));
      }
      if (check.noHTML && data.value.match(/<[\s\S]*>/)) {
        return Promise.reject(new _Error2.default(this, data, 'No tags allowed in this text.'));
      }
    }
    return Promise.resolve();
  }

  min(limit) {
    const set = this._setting;
    if (limit) {
      if (!(limit instanceof _Reference2.default)) {
        if (set.max && !this._isReference('max') && limit > set.max) {
          throw new Error('Min length can´t be greater than max length');
        }
        if (limit < 0) throw new Error('Length for min() has to be positive');
      }
      set.min = limit;
    } else delete set.min;
    return this;
  }

  max(limit) {
    const set = this._setting;
    if (limit) {
      if (!(limit instanceof _Reference2.default)) {
        if (set.min && !this._isReference('min') && limit < set.min) {
          throw new Error('Max length can´t be less than min length');
        }
        if (limit < 0) throw new Error('Length for max() has to be positive');
      }
      set.max = limit;
    } else delete set.max;
    return this;
  }

  length(limit) {
    const set = this._setting;
    if (limit) {
      if (!(limit instanceof _Reference2.default)) {
        if (limit < 0) throw new Error('Length has to be positive');
      }
      set.min = limit;
      set.max = limit;
    } else {
      delete set.min;
      delete set.max;
    }
    return this;
  }

  truncate(flag) {
    return this._setFlag('truncate', flag);
  }

  pad(side = 'right', char = ' ') {
    const set = this._setting;
    if (side === false) delete set.pad;else set.pad = new Pad(side, char);
    return this;
  }

  _lengthDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.min instanceof _Reference2.default) {
      msg += `Minimum character length depends on ${set.min.description}. `;
    }
    if (set.max instanceof _Reference2.default) {
      msg += `Maximum character length depends on ${set.max.description}. `;
    }
    if (!this._isReference('min') && !this._isReference('max') && set.min && set.max) {
      msg = set.min === set.max ? `The string has to contain exactly ${set.min} characters. ` : `The string can have between ${set.min} and ${set.max} characters. `;
    } else if (!this._isReference('min') && set.min) {
      msg = `The string needs at least ${set.min} characters. `;
    } else if (!this._isReference('max') && set.max) {
      msg = `The string allows up to ${set.min} characters. `;
    }
    if (set.truncate instanceof _Reference2.default) {
      msg += `Too long string will be truncated depending on ${set.truncate.description}. `;
    } else if (!this._isReference('truncate') && set.truncate) {
      msg += 'If it´s too long the string will be truncated. ';
    }
    if (set.pad) {
      msg += `If it´s too short the string will be padded on \
 ${set.pad.type === 'both' ? '' : 'the '}${set.pad.type} \
 ${set.pad.type === 'both' ? 'sides' : 'side'} using \`${_util2.default.inspect(set.pad.char)}\`. `;
    }
    return msg.length ? `${msg.trim()}\n` : msg;
  }

  _lengthValidator(data) {
    const check = this._check;
    try {
      this._checkNumber('min');
      this._checkNumber('max');
      this._checkBoolean('truncate');
      if (check.max && check.min && check.min > check.max) {
        throw new Error('Min length can´t be greater than max length');
      }
    } catch (err) {
      return Promise.reject(new _Error2.default(this, data, err.message));
    }

    let num = data.value.length;

    if (check.pad && num < check.min) {
      const add = check.min - num;
      let pad = check.pad.char;
      let a;
      switch (check.pad.type) {
        case 'right':
          if (pad.length < add) pad += pad.slice(-1).repeat(add - pad.length);
          data.value += pad.slice(-add);
          break;
        case 'left':
          if (pad.length < add) pad = `${pad.slice(0, 1).repeat(add - pad.length)}${pad}`;
          data.value = `${pad.slice(0, add)}${data.value}`;
          break;
        default:
          a = Math.ceil(add / 2);
          pad = check.pad.char.length > 1 ? check.pad.char.slice(-Math.ceil(check.pad.char.length / 2)) : check.pad.char;
          if (pad.length < a) pad += pad.slice(-1).repeat(a - pad.length);
          data.value += pad.slice(-a);
          pad = check.pad.char.length > 1 ? check.pad.char.slice(0, Math.ceil(check.pad.char.length / 2)) : check.pad.char;
          a = Math.floor(add / 2);
          if (pad.length < a) pad = `${pad.slice(0, 1).repeat(a - pad.length)}${pad}`;
          data.value = `${pad.slice(0, a)}${data.value}`;
      }
      num = data.value.length;
    }

    if (check.truncate && num > check.max) {
      data.value = data.value.substr(0, check.max);
      num = data.value.length;
    }

    if (check.min && num < check.min) {
      return Promise.reject(new _Error2.default(this, data, `The string has a length of ${num} characters. \
 This is too less, at least ${check.min} are needed.`));
    }
    if (check.max && num > check.max) {
      return Promise.reject(new _Error2.default(this, data, `The string has a length of ${num} characters. \
 This is too much, not more than ${check.max} are allowed.`));
    }
    return Promise.resolve();
  }

  match(re) {
    const set = this._setting;
    if (re === undefined) delete set.match;else {
      if (!set.match) set.match = [];
      set.match.push(re);
    }
    return this;
  }

  notMatch(re) {
    const set = this._setting;
    if (re === undefined) delete set.notMatch;else {
      if (!set.notMatch) set.notMatch = [];
      set.notMatch.push(re);
    }
    return this;
  }

  _matchDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.match && set.match.length || set.notMatch && set.notMatch.length) {
      msg += 'The text should:';
      if (set.match && set.match.length) {
        msg += set.match.map(e => {
          if (e instanceof _Reference2.default) return `\n- match ${e.description}`;
          return `\n- match \`${_util2.default.inspect(e)}\``;
        }).join('');
      }
      if (set.notMatch && set.notMatch.length) {
        msg += set.notMatch.map(e => {
          if (e instanceof _Reference2.default) return `\n- not match ${e.description}`;
          return `\n- not match \`${_util2.default.inspect(e)}\``;
        }).join('');
      }
      msg += '\n\n';
    }
    return msg;
  }

  _matchValidator(data) {
    const check = this._check;
    try {
      this._checkArrayMatch('match');
      this._checkArrayMatch('notMatch');
    } catch (err) {
      return Promise.reject(new _Error2.default(this, data, err.message));
    }

    if (check.match && check.match.length) {
      const fail = check.match.filter(e => {
        if (typeof e === 'string') return !data.value.includes(e);
        return !data.value.match(e);
      }).map(e => `\`${_util2.default.inspect(e)}\``).join(', ').replace(/(.*), /, '$1 and ');
      if (fail) {
        return Promise.reject(new _Error2.default(this, data, `The text should match: ${fail}`));
      }
    }
    if (check.notMatch && check.notMatch.length) {
      const fail = check.notMatch.filter(e => {
        if (typeof e === 'string') return data.value.includes(e);
        return data.value.match(e);
      }).map(e => `\`${_util2.default.inspect(e)}\``).join(', ').replace(/(.*), /, '$1 and ');
      if (fail) {
        return Promise.reject(new _Error2.default(this, data, `The text should not match: ${fail}`));
      }
    }
    return Promise.resolve();
  }
}

exports.default = StringSchema;