'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _String = require('./String');

var _String2 = _interopRequireDefault(_String);

var _Domain = require('./Domain');

var _Domain2 = _interopRequireDefault(_Domain);

var _Error = require('../Error');

var _Error2 = _interopRequireDefault(_Error);

var _Reference = require('../Reference');

var _Reference2 = _interopRequireDefault(_Reference);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class URLSchema extends _String2.default {
  constructor(base) {
    super(base);

    let raw = this._rules.descriptor.pop();
    let allow = this._rules.descriptor.pop();
    this._rules.descriptor.push(this._structDescriptor, this._resolveDescriptor, allow, this._existsDescriptor, raw);
    raw = this._rules.validator.pop();
    allow = this._rules.validator.pop();
    this._rules.validator.push(this._structValidator, this._resolveValidator, allow, this._existsValidator, this._formatValidator, raw);
  }

  stripEmpty() {
    return this._setError('stripEmpty');
  }
  truncate() {
    return this._setError('truncate');
  }
  pad() {
    return this._setError('pad');
  }

  dns(flag) {
    return this._setFlag('dns', flag);
  }

  _typeDescriptor() {
    return 'It has to be a valid URI address.\n';
  }

  _structDescriptor() {
    const set = this._setting;
    const schema = new _Domain2.default();
    if (set.dns) schema.dns();
    return `- domain part: ${schema.description}\n\n`;
  }

  _structValidator(data) {
    const check = this._check;

    data.value = _url2.default.parse(data.value);

    const schema = new _Domain2.default().raw();
    if (check.dns) schema.dns();
    return schema._validate(data.sub('host'));
  }

  resolve(base) {
    return this._setAny('resolve', base);
  }

  _resolveDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.resolve) {
      if (this._isReference('resolve')) {
        msg += `A relative address is based on the address defined under \
${set.resolve.description}. `;
      } else msg += `A relative address is based on ${set.resolve}. `;
    }
    return msg.length ? `${msg.trim()}\n` : msg;
  }

  _resolveValidator(data) {
    const check = this._check;
    try {
      this._checkString('resolve');
    } catch (err) {
      return Promise.reject(new _Error2.default(this, data, err.message));
    }

    if (check.resolve) data.value = _url2.default.resolve(check.resolve, data.value);
    return Promise.resolve();
  }

  _allowValidator(data) {
    const check = this._check;
    this._checkArrayString('allow');
    this._checkArrayString('deny');

    if (check.deny && check.deny.length) {
      const list = check.deny.map(e => _url2.default.parse(e));
      const protocol = list.map(e => e.protocol).filter(e => e && e.length && data.value.protocol === e);
      const host = list.map(e => e.host).filter(e => e && e.length && data.value.host === e);
      const hostname = list.map(e => e.hostname).filter(e => e && e.length && data.value.hostname === e);
      const path = list.map(e => e.path).filter(e => e && e.length && data.value.path.includes(e));
      if (protocol.length || hostname.length || host.length || path.length) {
        return Promise.reject(new _Error2.default(this, data, 'URL found in blacklist (denied item).'));
      }
    }
    if (check.allow && check.allow.length) {
      const list = check.allow.map(e => _url2.default.parse(e));
      const protocol = list.map(e => e.protocol).filter(e => !e || !e.length || data.value.protocol === e);
      const host = list.map(e => e.host).filter(e => !e || !e.length || data.value.host === e);
      const hostname = list.map(e => e.hostname).filter(e => !e || !e.length || data.value.hostname === e);
      const path = list.map(e => e.path).filter(e => !e || !e.length || data.value.path.includes(e));
      if (!protocol.length || !hostname.length || !host.length || !path.length) {
        return Promise.reject(new _Error2.default(this, data, 'URL not found in whitelist (allowed item).'));
      }
    }
    return Promise.resolve();
  }

  exists(flag) {
    return this._setFlag('exists', flag);
  }

  _existsDescriptor() {
    const set = this._setting;
    let msg = '';
    if (set.exists) {
      if (this._isReference('exists')) {
        msg += `The URL have to exist and be accessible if defined under \
${set.exists.description}. `;
      } else msg += 'The URL have to exist and be accessible. ';
    }
    return msg.length ? `${msg.trim()}\n` : msg;
  }

  _existsValidator(data) {
    const check = this._check;
    try {
      this._checkBoolean('exists');
    } catch (err) {
      return Promise.reject(new _Error2.default(this, data, err.message));
    }

    if (check.exists) {
      return Promise.resolve().then(() => require('request-promise-native')).then(request => request(data.value.href));
    }
    return Promise.resolve();
  }

  _formatValidator(data) {
    data.value = data.value.href;
    return Promise.resolve();
  }
}

exports.default = URLSchema;